import usb
import time
import sys


class MissileLauncher:
    DOWN  = 0x01
    UP    = 0x02
    LEFT  = 0x04
    RIGHT = 0x08
    FIRE  = 0x10
    STOP  = 0x20

    DELAY = 0.1
    
    def __init__(self):
        self.dev = usb.core.find(idVendor=0x2123)
        if self.dev.is_kernel_driver_active(self.dev[0][0,0]):
            self.dev.detach_kernel_driver(self.dev[0][0,0])
        self.dev.set_configuration()

    def _send_cmd(self,cmd):
            self.dev.ctrl_transfer(0x21, 0x09, 0,0, [0x02, cmd, 00, 00, 00, 00, 00, 00])
        
    def move_left(self):
        self._send_cmd(MissileLauncher.LEFT)

    def move_right(self):
        self._send_cmd(MissileLauncher.RIGHT)

    def move_up(self):
        self._send_cmd(MissileLauncher.UP)

    def move_down(self):
        self._send_cmd(MissileLauncher.DOWN)

    def fire(self):
        self._send_cmd(MissileLauncher.FIRE)

    def led_on(self):
        self.dev.ctrl_transfer(0x21, 0x09, 0,0, [0x03, 0x01, 00, 00, 00, 00, 00, 00])
        
    def led_off(self):
        self.dev.ctrl_transfer(0x21, 0x09, 0,0, [0x03, 0x00, 00, 00, 00, 00, 00, 00])

    def get_limit_reached(self):
        print dev.ctrl_transfer(0xa1,0x01,0x0307,0x03,0x13)

    def read(self):
        self.dev.ctrl_transfer(0x21,0x09, 0, 0, [0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00])
        return self.dev[0][0,0][0].read(8)

    def stop(self):
        self._send_cmd(MissileLauncher.STOP)

    def center_horizontal(self):
        self.move_right()
        while not (self.read()[1]&self.RIGHT):
            time.sleep(.05)
        self.stop()

        p=self.read()[1]

        if p&self.RIGHT: #to the max right? move left and time it
            t1=time.time()
            self.move_left()
            while not (self.read()[1]&self.LEFT):
                pass
            self.stop()
            t2=time.time()

            #move to the middle (half way)
            self.move_right()
            time.sleep((t2-t1)/2.0)
            self.stop()

    def center_vertical(self):
        self.move_down()
        while not (self.read()[1]&self.DOWN):
            time.sleep(.02)
        self.stop()

        p=self.read()[1]
        if p&self.DOWN: #to the max down?
            self.move_up()
            t1=time.time()
            while not (self.read()[1]&self.UP):
                pass
            self.stop()
            t2=time.time()

            #move to the middle
            self.move_down()
            time.sleep((t2-t1)/2.0)
            self.stop()


if __name__=='__main__':
    m=MissileLauncher()
    m.led_on()

    m.center_horizontal()
    m.center_vertical()

    m.led_off()
